package vn.tmhtechlab.flutter_nearby_messages_api;

import androidx.annotation.NonNull;
import android.app.Activity;
import android.util.Log;

import com.google.android.gms.nearby.Nearby;
import com.google.android.gms.nearby.messages.Message;
import com.google.android.gms.nearby.messages.MessageListener;
import com.google.android.gms.nearby.messages.PublishOptions;
import com.google.android.gms.nearby.messages.Strategy;
import com.google.android.gms.nearby.messages.SubscribeOptions;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;

import static android.content.ContentValues.TAG;

public class FlutterNearbyMessagesApiPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  private Activity activity;
  private MethodChannel channel;
  private Message mActiveMessage;
  private MessageListener mMessageListener;

  // STATUS OF PUBLISH MESSAGE AND SUBSCRIBE
  static final int ACTIVED = 2;
  static final int INACTIVE = 0;

  static final String ON_FOUND = "onFound";
  static final String ON_LOST = "onLost";
  static final String STATUS_HANDLER = "statusHandler";

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
    channel = new MethodChannel(binding.getBinaryMessenger(), "flutter_nearby_messages_api");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    activity = null;
    mActiveMessage = null;
    mMessageListener = null;
  }

  /**
   * background receiver subscription
   */
  void backgroundSubscribe() {
    backgroundUnsubscribe();
    initMessageListener();

    Strategy strategy = new Strategy.Builder().setTtlSeconds(Strategy.TTL_SECONDS_DEFAULT)
        .setDistanceType(Strategy.DISTANCE_TYPE_DEFAULT).setDiscoveryMode(Strategy.DISCOVERY_MODE_DEFAULT).build();

    SubscribeOptions options = new SubscribeOptions.Builder().setStrategy(strategy).build();
    Nearby.getMessagesClient(activity).subscribe(mMessageListener, options)
        .addOnSuccessListener(activity, new OnSuccessListener<Void>() {
          @Override
          public void onSuccess(Void aVoid) {
            Log.i(TAG, "backgroundSubscribe: OnSuccess");

          }
        }).addOnCompleteListener(activity, new OnCompleteListener<Void>() {
          @Override
          public void onComplete(@NonNull Task<Void> task) {
            channel.invokeMethod(STATUS_HANDLER, String.valueOf(ACTIVED));
            Log.i(TAG, "backgroundSubscribe: onComplete");
          }
        });
  }

  void initMessageListener() {
    mMessageListener = new MessageListener() {
      @Override
      public void onFound(Message message) {
        String msg = new String(message.getContent());
        channel.invokeMethod(ON_FOUND, msg);
      }

      @Override
      public void onLost(Message message) {
        String msg = new String(message.getContent());
        channel.invokeMethod(ON_LOST, msg);
      }
    };
  }

  /**
   * unsubscribe from broadcast receiver
   */
  void backgroundUnsubscribe() {
    Log.i(TAG, "Unsubscribe!");

    if (mMessageListener != null) {
      Nearby.getMessagesClient(activity).unsubscribe(mMessageListener);
      mMessageListener = null;
      channel.invokeMethod(STATUS_HANDLER, String.valueOf(INACTIVE));
    }
  }

  void publish(String message) {
    unPublish();

    mActiveMessage = new Message(message.getBytes());
    Strategy strategy = new Strategy.Builder().setTtlSeconds(Strategy.TTL_SECONDS_DEFAULT)
        .setDistanceType(Strategy.DISTANCE_TYPE_DEFAULT).setDiscoveryMode(Strategy.DISCOVERY_MODE_DEFAULT).build();
    PublishOptions options = new PublishOptions.Builder().setStrategy(strategy).build();
    Nearby.getMessagesClient(activity).publish(mActiveMessage, options)
        .addOnSuccessListener(activity, new OnSuccessListener<Void>() {
          @Override
          public void onSuccess(Void aVoid) {
            Log.i(TAG, "OnSuccess...");
            channel.invokeMethod(STATUS_HANDLER, String.valueOf(ACTIVED));
          }
        }).addOnCompleteListener(activity, new OnCompleteListener<Void>() {
          @Override
          public void onComplete(@NonNull Task<Void> task) {
            Log.i(TAG, "onComplete...");
          }
        });
  }

  void unPublish() {
    Log.i(TAG, "UnPublish!");

    if (mActiveMessage != null) {
      Nearby.getMessagesClient(activity).unpublish(mActiveMessage);
      mActiveMessage = null;
      channel.invokeMethod(STATUS_HANDLER, String.valueOf(INACTIVE));
    }
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding) {
    activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
    activity = null;
  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
    activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivity() {
    activity = null;
  }

  /**
   * Host will invoke these methods to set up the nearby messages API listener
   * 
   * @param call
   * @param result
   */
  @Override
  public void onMethodCall(MethodCall call, Result result) {
    switch (call.method) {
      case "setAPIKey":
        Log.i(TAG,
            "~~~ Only on iOS. Please reference https://developers.google.com/nearby/messages/android/get-started");

        result.success(null);
        break;
      case "backgroundSubscribe":
        this.backgroundSubscribe();
        result.success(null);
        break;
      case "backgroundUnsubscribe":
        this.backgroundUnsubscribe();
        result.success(null);
        break;
      case "publish":
        final String message = call.argument("message");

        Log.i(TAG, "Message is: " + message);
        this.publish(message);
        result.success(null);
        break;
      case "unPublish":
        this.unPublish();
        result.success(null);
        break;
      case "setPermissionAlert":
        Log.i(TAG, "~~~ setPermissionAlert: Only on iOS");
        result.success(null);
        break;
      case "setNearbyAccessPermission":
        Log.i(TAG, "~~~ setNearbyAccessPermission: Only on iOS");
        result.success(null);
        break;
      case "enableDebugMode":
        Log.i(TAG, "~~~ enableDebugMode: Available on iOS. Android is TODO");
        result.success(null);
        break;
      case "disableDebugMode":
        Log.i(TAG, "~~~ disableDebugMode: Available on iOS. Android is TODO");
        result.success(null);
        break;
      default:
        result.notImplemented();
    }
  }
}