//
//  NearbyMessagesApiMCMs.m
//  nearby_messages_api
//
//  Created by Thomas Cross on 3/2/19.
//

#import "NearbyMessagesApiMCMs.h"
#import <Flutter/Flutter.h>
#import "GNSMessages.h"

static NSString * const kOnFound = @"onFound";
static NSString * const kOnLost = @"onLost";
static NSString * const kStatusHandler = @"statusHandler";
static NSString * const kPermissionHandler = @"permissionHandler";
static NSString * const kMicrophonePermissionErrorHandler = @"microphonePermissionErrorHandler";
static NSString * const kBluetoothPowerErrorHandler = @"bluetoothPowerErrorHandler";
static NSString * const kBluetoothPermissionErrorHandler = @"bluetoothPermissionErrorHandler";

typedef NS_ENUM(NSUInteger, GNSPermissionStatus) {
  kDeny,
  kGranted
};

@interface NearbyMessagesApiMCMs ()
@property(nonatomic) GNSPermission *nearbyPermission;
@property(nonatomic) FlutterMethodChannel *channel;
@end

@implementation NearbyMessagesApiMCMs

// Private
GNSPermission *nearbyPermission;
GNSMessageManager *messageMgr;
GNSStrategy *strategy;
id<GNSPublication> publication;
id<GNSSubscription> subscription;

// Custom Alert
BOOL useNearbyDefaultAlert = true;
NSString *titleNearbyPermissionAlert;
NSString *messageNearbyPermissionAlert;
NSString *denyTitle;
NSString *grantedTitle;


NSString * const GNSOperationStatusCategories[] = {
    [kGNSOperationStatusInactive] = @"0",
    [kGNSOperationStatusStarting] = @"1",
    [kGNSOperationStatusActive] = @"2"
};

NSString * const GNSPermissionStatusCategories[] = {
    [kDeny] = @"0",
    [kGranted] = @"1"
};

- (instancetype) init {
    return [self initWithChannel:nil];
}

- (instancetype) initWithChannel:(FlutterMethodChannel *)channel {
    self = [super init];
    if (self) {
        self.channel = channel;
    }
    return self;
}

- (void) setAPIKey:(NSString *)apikey {
    messageMgr = [[GNSMessageManager alloc]
                  initWithAPIKey:apikey
                  paramsBlock:^(GNSMessageManagerParams *params) {
        params.microphonePermissionErrorHandler = ^(BOOL hasError) {
            if (hasError) {
                [self.channel invokeMethod:kMicrophonePermissionErrorHandler arguments:[self getGNSOperationStatusString:kGNSOperationStatusInactive]];
            }
        };
        params.bluetoothPowerErrorHandler = ^(BOOL hasError) {
            if (hasError) {
                [self.channel invokeMethod:kBluetoothPowerErrorHandler arguments:[self getGNSOperationStatusString:kGNSOperationStatusInactive]];
            }
        };
        params.bluetoothPermissionErrorHandler = ^(BOOL hasError) {
            if (hasError) {
                [self.channel invokeMethod:kBluetoothPermissionErrorHandler arguments:[self getGNSOperationStatusString:kGNSOperationStatusInactive]];
            }
        };
    }];
    
    [self listenerPermission];
    [self setStrategy];
}

// Private
- (void) setStrategy {
    strategy = [GNSStrategy strategyWithParamsBlock:^(GNSStrategyParams *params) {
        params.allowInBackground = NO;
        params.discoveryMediums = kGNSDiscoveryMediumsDefault;
        params.discoveryMode = kGNSDiscoveryModeDefault;
    }];
}

- (void) setNearbyAccessPermission:(BOOL)value {
    [GNSPermission setGranted:value];
}

/// Starts sharing with a randomized name and scanning for others.
- (void) backgroundSubscribe {
    [self backgroundUnsubscribe];
    
    // Subscribe to messages from nearby devices and display them in the message view.
    NSString *(^stringFromData)(NSData *) = ^(NSData *content) {
        return [[NSString alloc] initWithData:content encoding:NSUTF8StringEncoding];
    };
    subscription = [messageMgr
                    subscriptionWithMessageFoundHandler:^(GNSMessage *message) {
        NSString *messageString = stringFromData(message.content);
        
        [self.channel invokeMethod:kOnFound arguments:messageString];
    }
                    messageLostHandler:^(GNSMessage *message) {
        NSString *messageString = stringFromData(message.content);
        
        [self.channel invokeMethod:kOnLost arguments:messageString];
    } paramsBlock:^(GNSSubscriptionParams *params) {
        params.strategy = strategy;
        
        params.statusHandler = ^(GNSOperationStatus status) {
            NSString *statusString = [self getGNSOperationStatusString:status];
            [self.channel invokeMethod:kStatusHandler arguments:statusString];
        };
        
        [self handlerNearbySubscribePermission:params];
    }];
}

- (void) permissionAlert:(GNSPermissionHandler)permissionHandler {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:titleNearbyPermissionAlert
                                                                   message:messageNearbyPermissionAlert
                                                            preferredStyle:(UIAlertControllerStyleAlert)];
    UIAlertAction *denyAction = [UIAlertAction actionWithTitle:denyTitle
                                                           style:UIAlertActionStyleCancel
                                                         handler:^(UIAlertAction * _Nonnull action) {
        [self.channel invokeMethod:kPermissionHandler arguments:[self getGNSPermissionStatusString:kDeny]];
        
        permissionHandler(NO);
    }];
    
    [alert addAction:denyAction];
    
    UIAlertAction *grantAction = [UIAlertAction actionWithTitle:grantedTitle
                                                       style:UIAlertActionStyleDefault
                                                     handler:^(UIAlertAction * _Nonnull action) {
        [self.channel invokeMethod:kPermissionHandler arguments:[self getGNSPermissionStatusString:kGranted]];
        
        permissionHandler(YES);
    }];
    
    [alert addAction:grantAction];
    
    [[UIApplication sharedApplication].keyWindow.rootViewController presentViewController:alert
                                                                                 animated:YES
                                                                               completion:^{}];
}

- (void) handlerNearbySubscribePermission:(GNSSubscriptionParams *) params {
    if (useNearbyDefaultAlert) { return; }
    
    if (![GNSPermission isGranted]) {
        params.permissionRequestHandler = ^(GNSPermissionHandler permissionHandler) {
            // Show your custom dialog here.
            // Don't forget to call permissionHandler() with YES or NO when the user dismisses it. (IMPORTANT)
            [self permissionAlert:permissionHandler];
        };
    }
}

- (void) handlerNearbyPublishPermission:(GNSPublicationParams *) params {
    if (useNearbyDefaultAlert) { return; }
    
    if (![GNSPermission isGranted]) {
        params.permissionRequestHandler = ^(GNSPermissionHandler permissionHandler) {
            // Show your custom dialog here.
            // Don't forget to call permissionHandler() with YES or NO when the user dismisses it. (IMPORTANT)
            [self permissionAlert:permissionHandler];
        };
    }
}

- (void) publish:(NSString *) message {
    [self unPublish];
    
    // Publish the name to nearby devices.
    GNSMessage *pubMessage = [GNSMessage messageWithContent:[message dataUsingEncoding:NSUTF8StringEncoding]];
    
    publication = [messageMgr publicationWithMessage:pubMessage paramsBlock:^(GNSPublicationParams *params) {
        params.statusHandler = ^(GNSOperationStatus status) {
            NSString *statusString = [self getGNSOperationStatusString:status];
            [self.channel invokeMethod:kStatusHandler arguments:statusString];
        };
        
        [self handlerNearbyPublishPermission:params];

        params.strategy = strategy;
    }];
}

- (void) unPublish {
    publication = nil;
}

- (void) backgroundUnsubscribe {
    subscription = nil;
}

- (void) enableDebugMode {
    [GNSMessageManager setDebugLoggingEnabled:YES];
}

- (void) disableDebugMode {
    [GNSMessageManager setDebugLoggingEnabled:NO];
}

- (NSString *) getGNSOperationStatusString:(GNSOperationStatus)status {
    return GNSOperationStatusCategories[status];
}

- (NSString *) getGNSPermissionStatusString:(GNSPermissionStatus)status {
    return GNSPermissionStatusCategories[status];
}

- (void) listenerPermission {
    nearbyPermission = [[GNSPermission alloc] initWithChangedHandler:^(BOOL granted) {
        // Update the UI here
    }];
}

- (void) setPermissionAlert:(NSString *)titleText
                          :(NSString *)messageText
                          :(NSString *)denyText
                          :(NSString *)grantText {
    useNearbyDefaultAlert = NO;
    titleNearbyPermissionAlert = titleText;
    messageNearbyPermissionAlert = messageText;
    denyTitle = denyText;
    grantedTitle = grantText;
}

@end
